#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Thu Feb 20 15:12:09 2020

@author: marisaeisenberg
"""

import pycxsimulator
from pylab import *
from copy import deepcopy

import networkx as nx

p_i = 0.5 # infection probability per contact
p_r = 0.5 # recovery probability
p_0 = 0.5 # probability of breaking tie if infected neighbor
p_1 = 0.1 # probability of regrowing tie if non-infected

prev = []
lostedges = []


def initialize():
    global g, nextg, prev
    g = nx.karate_club_graph()
    # g = nx.barabasi_albert_graph(50,5)
    g.pos = nx.spring_layout(g)
    nx.set_node_attributes(g, 0, 'state')  # everyone starts off susceptible
    g.node[1]['state'] = 1                 # set one node to be infected (index case)
    nextg = g.copy()
    nextg.pos = g.pos
    prev.append(1/len(g.nodes)) # initial prevalence

def update():
    global g, nextg, prev
    curprev = 0
    nextg = deepcopy(g)
    for a in g.nodes:
        if g.node[a]['state'] == 0: # if susceptible
            nextg.node[a]['state'] = 0
            for b in g.neighbors(a):
                if g.node[b]['state'] == 1: # if neighbor b is infected
                    if random() < p_0:  # with probability p_0 cut them off at next time step
                        nextg.remove_edge(a, b)
                        lostedges.append([a,b])
                    if random() < p_i: # but we still let them potentially transmit now
                        nextg.node[a]['state'] = 1                    
        else: # if infected
            curprev += 1
            nextg.node[a]['state'] = 0 if random() < p_r else 1
    
    for i in range(len(lostedges)): # go through dropped edges and see which ones we can get back
        if g.node[lostedges[i][0]]['state'] == 0 and g.node[lostedges[i][1]]['state'] == 0:
            if random() < p_1:
                nextg.add_edge(lostedges[i][0],lostedges[i][1])
    
    prev.append(curprev/len(g.nodes()))
    g = deepcopy(nextg)
    
    
def observe():
    global g, prev
    cla()
    nx.draw(g, cmap = cm.Wistia, vmin = 0, vmax = 1,
            node_color = [g.node[i]['state'] for i in g.nodes],
            pos = g.pos)

pycxsimulator.GUI().start(func=[initialize, observe, update])


epicurve = scatter(range(len(prev)), prev)
xlabel("Time")
ylabel("Prevalence")
show(epicurve)





